local DEBUG = false

---------------------
-- Version
local RELEASETYPE = "Release"
local VERSION = GetAddOnMetadata("RaidWatch", "Version") or "1.0"
local REVISION = "294"

--@alpha@
RELEASETYPE = "Alpha"
--@end-alpha@

if REVISION:find("@") then 
	REVISION = "Dev" 
	RELEASETYPE = "Development"
	DEBUG = true
end

---------------------
-- Create Loader
RaidWatchLoD = LibStub("AceAddon-3.0"):NewAddon("RaidWatchLoD", "AceEvent-3.0", "AceConsole-3.0")
local RWL = RaidWatchLoD

---------------------
-- Locals
local L = LibStub("AceLocale-3.0"):GetLocale("RaidWatch2_Loader")
local loadOnZone = {}
local loadOnCore = {}

RW1 = loadOnCore
RW2 = loadOnZone

---------------------
-- Init
function RWL:OnInitialize()
	for i = 1, GetNumAddOns() do
		if GetAddOnMetadata(i, "X-RW-Mod") then
			local modId	= GetAddOnInfo(i)
			local name 	= GetAddOnMetadata(i, "X-RW-Name") or ""
			local icon = GetAddOnMetadata(i, "X-RW-Icon")
			
			if GetAddOnMetadata(i, "X-RW-Zone") then
				local zones = {strsplit(",", GetAddOnMetadata(i, "X-RW-Zone") or "")}
				for i, zone in pairs(zones) do
					loadOnZone[zone] = loadOnZone[zone] or {}
					loadOnZone[zone][modId] = {
						name = name,
						icon = icon,
					}
				end
			end
			
			if GetAddOnMetadata(i, "X-RW-LoadOnCore") then
				loadOnCore[modId] = {
					name = name,
					icon = icon,
				}
			end
		end
	end
end

---------------------
-- Enable
function RWL:OnEnable()
	if RELEASETYPE == "Alpha" then
		self:Msg(L.ALPHA_INFO)
	elseif RELEASETYPE == "Development" then
		self:Msg(L.DEV_INFO)
	end
	
	self:RegisterChatCommand("mrw", "SlashHandler")
	self:RegisterChatCommand("rw2", "SlashHandler")
	self:RegisterChatCommand("range", "ShowRange")
end

-- General message function, usualy embedded in plugins 
-- and bosses to automaticly supply their own prefix.
function RWL:Msg(msg, prefix)
	local prefix
	if self.L and rawget(self.L, "name") then
		prefix = rawget(self.L, "name")
	else
		prefix = prefix or self:GetName()
	end
	DEFAULT_CHAT_FRAME:AddMessage(("|cff28579d[|r|cffcfe846%s|r|cff28579d]|r "):format(tostring(prefix))..msg, 0.39, 0.67, 0.87)
end

--------------------------------------------
-- Slash commands
function RWL:SlashHandler(input)
	if input == "" or input == "config" then
		local config = self:GetPlugin("Options")
		if not config then	
			local loaded, reason = LoadAddOn("RW_Options")
			if loaded then
				config = self:GetPlugin("Options")
			else 
				self:Msg(L.COULD_NOT_LOAD_OPTIONS:format(_G["ADDON_"..reason]))
			end
		end
		if config then config:Toggle() end
	end
end

function RWL:ShowRange(input)
	local number = tonumber(input)
	if type(number) == "number" then
		self:SendMessage("RW_ShowRangeCheck", true, number)
	else
		self:SendMessage("RW_ShowRangeCheck", true, 11)
	end
end

---------------------------------------------
-- LDB
function RWL:SetUpLDB()
	local LDB = LibStub("LibDataBroker-1.1")
	if not LDB then return end
	local l = LDB:NewDataObject("RW2")
	l.type = "launcher"
	l.icon = [[Interface\Icons\Achievement_Dungeon_NexusRaid_Normal.blp]]
	l.tocname = "RW_Core"
	l.label = "Raid Watch 2"
	l.OnClick = function()
		RWL:SlashHandler("config")
	end
	l.OnTooltipShow = function(tt)
		tt:AddLine("|cffcfe846Raid Watch 2|r")
		tt:AddLine("Click to show settings")
	end
	local LibDBIcon = LibStub("LibDBIcon-1.0")
	if not LibDBIcon then return end
	LibDBIcon:Register("RW2", l, self.db.profile.minimap)
end