local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("DebugBoss")


local mods = {}
local trackers = {}
local yells = {}
local deaths = {}
plugin.mods = mods

local defaults = {
	profile = {
		srouce = "",
		target = "",
		amount = 0,
		event = "",
	}
}

local testEvents = {
	SPELL_AURA_APPLIED = "SPELL_AURA_APPLIED",
	SPELL_AURA_APPLIED_DOSE = "SPELL_AURA_APPLIED_DOSE",
	SPELL_AURA_REMOVED = "SPELL_AURA_REMOVED",
	SPELL_DAMAGE = "SPELL_DAMAGE",
	SPELL_CAST_START = "SPELL_CAST_START",
	SPELL_CAST_SUCCESS = "SPELL_CAST_SUCCESS",
	SPELL_HEAL = "SPELL_HEAL",
	SPELL_SUMMON = "SPELL_SUMMON",
	CHAT_MSG_MONSTER_YELL = "CHAT_MSG_MONSTER_YELL",
	CHAT_MSG_MONSTER_EMOTE = "CHAT_MSG_MONSTER_EMOTE",
	CHAT_MSG_RAID_BOSS_EMOTE = "CHAT_MSG_RAID_BOSS_EMOTE",
	CHAT_MSG_MONSTER_SAY = "CHAT_MSG_MONSTER_SAY",
}

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace(self:GetName(), defaults)
	self:RegisterMessage("RW_BossRegistered")
	self:RegisterMessage("RW_BossModFound")
	self:RegisterMessage("RW_TrackRegistered")
	self:RegisterMessage("RW_YellRegistered")
	self:RegisterMessage("RW_DeathRegistered")
end

function plugin:RW_BossRegistered(message, boss)
	tinsert(mods[boss.modPack], boss)
end

function plugin:RW_BossModFound(message, mod, id)
	mods[mod] = mods[mod] or {}
end

function plugin:RW_TrackRegistered(message, boss, func, ...)
	trackers[boss:GetName()] = trackers[boss:GetName()] or {}
	for i = 1, select("#", ...) do
		sp = select(i, ...)
		trackers[boss:GetName()][sp] = true
	end
end

function plugin:RW_YellRegistered(message, boss, func, ...)
	yells[boss:GetName()] = yells[boss:GetName()] or {}	
	for i = 1, select("#", ...) do
		msg = select(1, ...)
		yells[boss:GetName()][(type(func) == "string" and func or msg)] = msg
	end
end

function plugin:RW_DeathRegistered(message, boss, func, ...)
	deaths[boss:GetName()] = deaths[boss:GetName()] or {}
	deaths[boss:GetName()][func] = deaths[boss:GetName()][func] or {}
	for i = 1, select("#", ...) do
		cID = select(i, ...)
		tinsert(deaths[boss:GetName()][func], cID)
	end
end

function plugin:OnEnable()
end

function plugin:GetBossOptions(boss)
	self:SendMessage("RW_BossEngaged", boss)
	self:SendMessage("RW_BossEnded", boss, true)	
end

local function engage(info)
	local boss = RW:GetBoss(info[#info - 1])
	if not boss then return end
	plugin:SendMessage("RW_BossEngaged", boss, "Debug")
end

local function disengage(info)
	boss = RW:GetBoss(info[#info - 1])
	if not boss then return end
	RW:SendMessage("RW_BossEnded", boss, true, "combat")
end

local function getTrackers(boss)
	if not trackers[boss] then return {} end
	local t = {}
	local db = plugin.db.profile
	inserted = {}
	for spell in pairs(trackers[boss]) do
		local spellName = RW.SpellName[spell]
		if not inserted[spellName] then
			t[spellName] = {
				type = "execute",
				name = spellName,
				func = function() 
					args = {}
					args.dName = db.target
					args.sName = db.source
					args.spellID = spell
					args.amount = tonumber(db.amount)
					args.event = db.event
					args.spellName = "Debugg Spell"
					plugin:SendMessage("RW_TestEvent", db.event, args)
				end,
			}
		end
	end	
	return t
end

local function getYells(boss)
	local db = plugin.db.profile
	local t = {}
	local bossModule = RW:GetBoss(boss)
	if bossModule.yellWins then
		for msg in pairs(bossModule.yellWins) do
			t[msg] = {
				type = "execute",
				name = "WinYell: "..msg,
				func = function() 
					scanner = RW:GetPlugin("CombatScanner")
					scanner:MonsterCom(db.event, msg, db.source)
				end,
			}
		end
	end
	if not yells[boss] then return t end
	inserted = {}
	for func, msg in pairs(yells[boss]) do
		if not inserted[msg] then
			t[msg] = {
				type = "execute",
				name = (type(func) == "string" and func or "")..": "..msg,
				func = function() 
					scanner = RW:GetPlugin("CombatScanner")
					scanner:MonsterCom(db.event, msg, db.source)
				end,
			}
		end
	end	
	return t
end

local function getDeaths(boss)
	local t = {}
	local boddModule = RW:GetBoss(boss)
	if boddModule.mobWins then
		for mob in pairs(boddModule.mobWins) do
			if not t[mob] then
				t[tostring(mob)] = {
					type = "execute",
					name = "MobWin: "..tostring(mob),
					func = function() 
						RW:GetPlugin("CombatScanner"):UNIT_DIED(mob)
					end,
				}
			end
		end
	end
	if not deaths[boss] then return t end
	for func, cIDs in pairs(deaths[boss]) do
		for i, cID in pairs(cIDs) do
			t[tostring(cID)] = {
				type = "execute",
				name = (type(func) == "string" and func or "")..": "..tostring(cID),
				func = function() 
					RW:GetPlugin("CombatScanner"):UNIT_DIED(cID)
				end,
			}
		end
	end
	return t
end

function plugin:GetOptions()
	local db = self.db.profile
	self.category = "Debug"
	
	for mod in pairs(mods) do
		for i, boss in pairs(mods[mod]) do
			self:SendMessage("RW_BossEngaged", boss)
			self:SendMessage("RW_BossEnded", boss)
		end
	end
	
	local function getBosses(mod)
		local ret = {}
		for i, boss in pairs(mods[mod]) do
			ret[boss:GetName()] = {
				type = "group",
				name = boss:GetName(),
				args = {
					Engage = {
						type = "execute",
						name = "Engage",
						func = engage,
						order = 1,
					},
					Disengage = {
						type = "execute",
						name = "Disengage",
						func = disengage,
						order = 2,
					},
					splitter = {
						type = "header",
						name = "Info",
						order = 10,
					},
					Source = {
						type = "input",
						name = "Source",
						set = function(info, value) db.source = value end,
						get = function(info) return db.source end,
						order = 11
					},
					Target = {
						type = "input",
						name = "Target",
						set = function(info, value) db.target = value end,
						get = function(info) return db.target end,
						order = 12,
					},
					Amount = {
						type = "input",
						name = "Amount",
						set = function(info, value) db.amount = value end,
						get = function(info) return db.amount end,
						order = 13,
					},
					Event = {
						type = "select",
						name = "Event",
						width = "full",
						values = function() return testEvents end,
						set = function(info, value) db.event = value end,
						get = function(info) return db.event end,
						order = 14
					},
					Trackers = {
						type = "group",
						inline = true,
						name = "Trackers",
						order = 15,
						args = getTrackers(boss:GetName()),						
					},
					Yells = {
						type = "group",
						inline = true,
						name = "Yells",
						order = 16,
						args = getYells(boss:GetName()),						
					},
					Deathts = {
						type = "group",
						inline = true,
						name = "Deaths",
						order = 17,
						args = getDeaths(boss:GetName()),
					}
				}
			}
		end
		return ret
	end
	
	local function getZones()
		local ret = {}
		for mod in pairs(mods) do
			ret[mod] = {
				type = "group",
				name = mod,
				args = getBosses(mod)
			}
		end
		return ret
	end

	local options = {
		type = "group",
		name = "Boss Debug",
		args = getZones()
	}
	return 999, "Debug", options
end
